/*	$ DOOROOS: drv_buz.c v 0.1 11-12-03 21:47 $	*/

/*
 * Copyright (C) 2015	iBOSN systems
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: drv_buz.c
 *
 * Purpose: Start of JBOSN Operating System.
 *          make the utility for wave device driver
 *
 */
#include <types.h>
#include <thread.h>
#include <message.h>
#include <interrupt.h>
#include <device.h>
#include <error.h>

#include <time.h>

#include <panic.h>

#include <mach/hal_interrupt.h>
#include <mach/SAMA5D3X.h>
#include <mach/SAMA5D3Xbd.h>
#include <mach/gpio.h>

#include <lib/string.h>
#include <dev/buzdrv.h>

//==========================================================================
STATIC BOOL                     th_BuzDevice( UINT32 Arg );

#define BUZDEV_STACK_SIZE      (512)
STATIC UINT32                   BuzDevStack[BUZDEV_STACK_SIZE>>2];

STATIC DEVICECONFIG             BuzDeviceConfig ={"BUZ", 0, -1, NULL, NULL };

PUBLIC THREADCONFIG             BuzDev_Thread =
                                            {
                                                (UINT32)&BuzDeviceConfig,
                                                th_BuzDevice,
                                                (PBYTE)BuzDevStack,
                                                BUZDEV_STACK_SIZE,
                                                THREAD_PRIORITY_DRIVER+10
                                            };
//==========================================================================

STATIC UINT32                   Channel;
STATIC HANDLE                   hBuzSoftTimer;
#define BUZ_TIMER_DELAY         100
#define BUZ_TIMER_BITMAP        0x100

#define BUZ_CHANNEL                 2
#define BUZ_DEFAULT_FREQUENCY       (2730)

//==========================================================================

typedef struct _BUZ_INFO_
{
    INT32       nFreq;
    INT32       OnDuration;
    INT32       OffDuration;
    INT32       nCount;

    INT32       nRemainDuration;
    INT32       nCurrentCount;
    BOOL        bOnOff;
}BUZ_INFO, *BUZ_PINFO;

STATIC BUZ_INFO     BuzInfo;

//STATIC PBUZ_MELODY pBuzMelody = NULL;
STATIC INT32    nRemainDuration;
STATIC UINT32   BaseFreq = 0;

STATIC VOID
BuzInit( VOID )
{
    UINT32 dwDiv;
    UINT32 dwTcClks;
    UINT32 dwMaxTcClks;

DBGLINE(1,"\n");
// Configure PIO controllers to periph mode
    const struct pio_desc TIOA2_pio[] = 
    {
        {"TIOA2",     AT91C_PIN_PE(27),   0, PIO_DEFAULT, PIO_PERIPH_B},
    };
    
    /* PIO configuration */
    //PIO_Configure( &gPinTIOA0, PIO_LISTSIZE( gPinTIOA0 ) ) ;
    pio_setup(TIOA2_pio, sizeof(TIOA2_pio)/sizeof(struct pio_desc));
DBGLINE(1," BOARD_MCK = %d\n", BOARD_MCK);
DBGLINE(1," SAMA5D3_ID_TC0 = %x\n", SAMA5D3_ID_TC0);

    /* Enable peripheral clock. */
    dwMaxTcClks = PMC_SetPeriMaxClock(SAMA5D3_ID_TC0, BOARD_MCK);
    PMC_EnablePeripheral( SAMA5D3_ID_TC0 ) ;
DBGLINE(1," dwMaxTcClks = %d\n", dwMaxTcClks);
DBGLINE(1," AT91C_BASE_TC0 = %x\n", AT91C_BASE_TC0);

    /* Configure TC for a 1Hz frequency and trigger on RC compare. */
    TC_FindMckDivisor( 2730*2000, dwMaxTcClks, &dwDiv, &dwTcClks, dwMaxTcClks ) ;
DBGLINE(1," dwDiv = %d\n", dwDiv);
DBGLINE(1," dwTcClks = %d\n", dwTcClks);
DBGLINE(1," dwMaxTcClks = %d\n", dwMaxTcClks);
    TC_Configure( AT91C_BASE_TC0, BUZ_CHANNEL, dwTcClks | TC_CMR_CPCTRG | TC_CMR_WAVE | TC_CMR_ACPA_CLEAR | TC_CMR_ACPC_SET) ;
    //TC_Configure( AT91C_BASE_TC0, BUZ_CHANNEL, dwTcClks | TC_CMR_CPCTRG | TC_CMR_WAVE | TC_CMR_BCPB_CLEAR | TC_CMR_BCPC_SET) ;

    BaseFreq = dwMaxTcClks / dwDiv ;
    
    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA = ( dwMaxTcClks / dwDiv ) / (BUZ_DEFAULT_FREQUENCY*2) ;
    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB = ( dwMaxTcClks / dwDiv ) / BUZ_DEFAULT_FREQUENCY ;
    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC = ( dwMaxTcClks / dwDiv ) / BUZ_DEFAULT_FREQUENCY ;

    BaseFreq = dwMaxTcClks / dwDiv ;
    
    BuzInfo.nFreq = BUZ_DEFAULT_FREQUENCY;
    BuzInfo.OnDuration = 500;
    BuzInfo.OffDuration = 500;
    BuzInfo.nCount = 1;
    BuzInfo.nRemainDuration = 0;
    BuzInfo.nCurrentCount = 0;
    BuzInfo.bOnOff = FALSE;
    
	return;
}

STATIC VOID
BuzSet( UINT32 Freq )
{
    if( Freq == 0 )
    {
        TC_Stop( AT91C_BASE_TC0, BUZ_CHANNEL );
        return;
    }

    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA = ( BaseFreq ) / (Freq*2) ;
    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB = ( BaseFreq ) / Freq ;
    AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC = ( BaseFreq ) / Freq ;

DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC);
    
    /* Start the Timer */
    TC_Start( AT91C_BASE_TC0, BUZ_CHANNEL ) ;

#if 0
for( ;; )
{
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RA);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RB);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC = %d\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RC);
    
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_CMR = %x\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_CMR);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_SMMR = %x\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_SMMR);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_CV = %x\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_CV);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RAB = %x\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_RAB);
DBGLINE(1," AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_SR = %x\n", AT91C_BASE_TC0->TCB_CHANNEL[BUZ_CHANNEL].TC_SR);
DBGLINE(1," AT91C_BASE_TC0->TCB_BMR = %x\n", AT91C_BASE_TC0->TCB_BMR);
Sleep(10);
}
#endif


//    /* Set TIOA0 trigger */
//    ADC_SetTrigger( AT91C_BASE_TSADC, ADC_MR_TRGSEL_ADC_TRIG1 ) ;
//    ADC_SetTriggerMode (AT91C_BASE_TSADC, ADC_TRGR_TRGMOD_EXT_TRIG_ANY); /* External Trigger Any Edge */

	return;
}


STATIC BOOL
th_BuzDevice( UINT32 Arg )
{
    JB_RECVMESSAGE      Message;
    HANDLE              hPort;
    HANDLE              Handle = NULL;
    PDEVICECONFIG       pDeviceConfig;

    BUZ_MELODY *pBuzMelody = NULL;
    
    pDeviceConfig = (PDEVICECONFIG)Arg;
    
    hPort = PortGet( );    
    PortSetType( hPort, PORTTYPE_MESSAGE|PORTTYPE_INTERRUPT|PORTTYPE_SIGNAL );

    Channel = ChannelGet( hPort );

    pDeviceConfig->Name[3] = (UINT32)pDeviceConfig->Minor + '1';
    pDeviceConfig->Channel = Channel;
    DeviceRegister( (PDEVICECONFIG)pDeviceConfig );

    //InterruptRegister(pDeviceConfig->nInterrupt /*SYSINTR_WAV*/, hPort);
    BuzInit( );

    hBuzSoftTimer = DrvTimerCreate( Channel, BUZ_TIMER_DELAY, BUZ_TIMER_BITMAP );
    DrvTimerStart(hBuzSoftTimer);

    BuzSet( BuzInfo.nFreq );

	while(1)
	{
		MessageReceive( hPort, &Message);

		switch(Message.Type)
		{
		case MESSAGE_TYPE_SIGNAL:
		{
kprintf(" buz timer \n");
#if 1
            if( Message.Param & BUZ_TIMER_BITMAP )
            {
                if( pBuzMelody == NULL)
                {
                    BuzInfo.nRemainDuration -= BUZ_TIMER_DELAY;
                    if( BuzInfo.nRemainDuration <= 0)
                    {
                        BuzInfo.nCurrentCount--;
                        if( BuzInfo.nCurrentCount <= 0)
                        {
                            BuzSet(0);
                            if( hBuzSoftTimer != NULL )
                                DrvTimerStop(hBuzSoftTimer); 

//kprintf(" buz timer stop\n");
                        }
                        else
                        {
                            if( BuzInfo.bOnOff )
                            {
                                BuzSet(0);
                                BuzInfo.bOnOff = FALSE;
                                BuzInfo.nRemainDuration = BuzInfo.OffDuration;
                            }
                            else
                            {
                                BuzSet(BuzInfo.nFreq);
                                BuzInfo.bOnOff = TRUE;
                                BuzInfo.nRemainDuration = BuzInfo.OnDuration;
                            }
                        }
                    }
                }
                else
                {
                    nRemainDuration -= BUZ_TIMER_DELAY;
                    if( nRemainDuration < 0 )
                    {
                        if( pBuzMelody->OnDuration <= 0 )
                        {
                            pBuzMelody = NULL;
                            TimerStop(hBuzSoftTimer);
                        }
                        else
                        {
                            nRemainDuration = pBuzMelody->OnDuration;
                            BuzSet( pBuzMelody->nFreq );
                            pBuzMelody++;
                        }
                    }
                }
            }
#endif
            continue;
		}

		case MESSAGE_TYPE_INTERRUPT:
            continue;

		case MESSAGE_TYPE_DEVICE_READ:
        {
		    Message.Type = 0;
			break;
	    }

		case MESSAGE_TYPE_DEVICE_WRITE:
	    {
		    Message.Type = 0;
			break;
	    }

		case MESSAGE_TYPE_DEVICE_CONTROL:
		{
#if 1
            PIOCONTROL_PARAM    pIoControlParam;

            pIoControlParam = (PIOCONTROL_PARAM)Message.Param;
            switch(pIoControlParam->Cmd)
            {
            case IOCTRL_BUZ_MELODY_PLAY:
                pBuzMelody = (PBUZ_MELODY)pIoControlParam->pInBuf;

                if( pBuzMelody == NULL || pBuzMelody->OnDuration <= 0 || hBuzSoftTimer == NULL)
                {
                    Message.Type = FALSE;
                    break;
                }

                nRemainDuration = pBuzMelody->OnDuration;
                BuzSet( pBuzMelody->nFreq );
                DrvTimerStart(hBuzSoftTimer); 
                pBuzMelody++;
                
                Message.Type = TRUE;
                break;

            case IOCTRL_BUZ_SET_FREQ:
                //kprintf("IOCTRL_BUZ_SET_FREQ");
                BuzInfo.nFreq = (UINT32)((UINT32)(pIoControlParam->pInBuf) & 0xFFFF);
                Message.Type = TRUE;
                break;

            case IOCTRL_BUZ_SET_ONDURATION:
                //kprintf("IOCTRL_BUZ_SET_ONDURATION");
                BuzInfo.OnDuration = (UINT32)(pIoControlParam->pInBuf);
                Message.Type = TRUE;
                break;

            case IOCTRL_BUZ_SET_OFFDURATION:
                //kprintf("IOCTRL_BUZ_SET_OFFDURATION");
                BuzInfo.OffDuration = (UINT32)(pIoControlParam->pInBuf);
                Message.Type = TRUE;
                break;

            case IOCTRL_BUZ_SET_COUNT:
                //kprintf("IOCTRL_BUZ_SET_COUNT");
                BuzInfo.nCount = (UINT32)(pIoControlParam->pInBuf)<<1;
                Message.Type = TRUE;
                break;

            case IOCTRL_BUZ_START:
            {
                if( pBuzMelody != NULL )
                {
                    Message.Type = FALSE;
                    break;
                }
                BuzSet(BuzInfo.nFreq);

                BuzInfo.nRemainDuration = BuzInfo.OnDuration;
                BuzInfo.nCurrentCount = BuzInfo.nCount;
                BuzInfo.bOnOff = TRUE;

                if( hBuzSoftTimer != NULL )
                    DrvTimerStart(hBuzSoftTimer); 

kprintf(" buz start - \n");
                Message.Type = TRUE;
                break;
            }

            case IOCTRL_BUZ_STOP:
            {
                BuzSet(0);


                if( hBuzSoftTimer != NULL )
                    DrvTimerStop(hBuzSoftTimer); 

                pBuzMelody = NULL;

                Message.Type = TRUE;
                break;
            }

            default:
                Message.Type = FALSE;
                break;
            }
#endif
            break;
        }

		case MESSAGE_TYPE_DEVICE_OPEN:
        {
            //Handle = ChannelOpenHandleGetFrom( Channel );
            Handle = Channel;
            Message.Type = Handle;

            break;
        }

		case MESSAGE_TYPE_DEVICE_CLOSE:
        {
		    Handle = NULL;
		    Message.Type = TRUE;
			break;
	    }

        default:
            break;
		}

        MessageReply(&Message);

	}

	return TRUE;	
}




#include <moduleloader.h>

PUBLIC HANDLE 
DllMain( HANDLE hDll, UINT32 Reason, PVOID Reserved )
{
    HANDLE hThread = NULL;
    switch( Reason )
    {
    case DLL_PROCESS_ATTACH:
        hThread = ThreadCreate (&BuzDev_Thread);
    	break;
    
    case DLL_PROCESS_DETACH:
    	break;
    }

    return hThread;
}

